<?php
/**
 * @codingStandardsIgnoreStart
 */
class PHPUnit_Framework_TestCase extends \PHPUnit\Framework\TestCase
{
}

interface PHPUnit_Framework_Test extends \PHPUnit\Framework\Test
{
}

interface PHPUnit_Framework_TestListener extends \PHPUnit\Framework\TestListener
{
}
