/*! Icinga Web 2 | (c) 2015 Icinga Development Team | GPLv2+ */

// Print styles

@media print {
  #fileupload-frame-target,
  #header,
  #responsive-debug,
  #sidebar,
  .controls > .tabs,
  .controls .filter,
  .controls .limiter-control,
  .controls .pagination-control,
  .controls .selection-info,
  .controls .sort-control,
  .dontprint, // Compat only, use dont-print instead
  .dont-print {
    display: none;
  }

  #layout,
  #main,
  .controls {
    position: static;
  }

  .container {
    float: none !important;
    width: 100% !important;
  }
}
