/*! Icinga Web 2 | (c) 2015 Icinga Development Team | GPLv2+ */

#about {
  .about-social i {
    font-size: 1.7em;
    color: @text-color;
  }

  .about-social i:hover {
    color: @icinga-blue;
  }

  .about-links {
    margin-top: 2.5em;
    margin-bottom: 2.5em;
  }

  .about-links i {
    margin: 0.5em;
    padding: 0;
    font-size: 5em;
    color: @text-color;
  }

  .about-links i:hover {
    color: @icinga-blue;
  }

  .common-table th {
    text-align: center;
  }
}
