<div class="controls">
    <?= $tabs ?>
</div>
<div id="about" class="content content-centered">
    <?= $this->img('img/icinga-logo-big-dark.png', null, array('width' => 320)) ?>
    <dl class="name-value-list">
    <?php if (isset($version['appVersion'])): ?>
        <dt><?= $this->translate('Icinga Web 2 Version') ?></dt>
        <dd><?= $this->escape($version['appVersion']) ?></dd>
    <?php endif ?>
    <?php if (isset($version['gitCommitID'])): ?>
        <dt><?= $this->translate('Git commit') ?></dt>
        <dd><?= $this->escape($version['gitCommitID']) ?></dd>
    <?php endif ?>
        <dt><?= $this->translate('PHP Version') ?></dt>
        <dd><?= $this->escape(PHP_VERSION) ?></dd>
    <?php if (isset($version['gitCommitDate'])): ?>
        <dt><?= $this->translate('Git commit date') ?></dt>
        <dd><?= $this->escape($version['gitCommitDate']) ?></dd>
    <?php endif ?>
        <dt><?= $this->translate('Copyright') ?></dt>
        <dd>
            <span>&copy; 2013-<?= date('Y') ?></span>
            <?= $this->qlink(
                $this->translate('The Icinga Project'),
                'https://icinga.com',
                null,
                array(
                    'target' => '_blank'
                )
            ) ?>
        </dd>
    </dl>
    <div class="about-social">
        <?= $this->qlink(
            null,
            'https://www.twitter.com/icinga',
            null,
            array(
                'target'    => '_blank',
                'icon'      => 'twitter',
                'title'     => $this->translate('Icinga on Twitter')
            )
        ) ?> <?= $this->qlink(
            null,
            'https://www.facebook.com/icinga',
            null,
            array(
                'target'    => '_blank',
                'icon'      => 'facebook-squared',
                'title'     => $this->translate('Icinga on Facebook')
            )
        ) ?>
    </div>
    <div class="about-links">
    <?= $this->qlink(
        null,
        'https://github.com/icinga/icingaweb2/issues',
        null,
        array(
            'target'    => '_blank',
            'icon'       => 'bug',
            'title'     => $this->translate('Report a bug')
        )
    ) ?>
    <?= $this->qlink(
        null,
        'https://icinga.com/support/',
        null,
        array(
            'target'    => '_blank',
            'icon'       => 'chat-empty',
            'title'     => $this->translate('Support')
        )
    ) ?>
        <?= $this->qlink(
            null,
            'https://icinga.com/community/',
            null,
            array(
                'target'    => '_blank',
                'icon'       => 'lightbulb',
                'title'     => $this->translate('Icinga Community')
            )
        ) ?>
        <?= $this->qlink(
            null,
            'https://docs.icinga.com/',
            null,
            array(
                'target'    => '_blank',
                'icon'       => 'doc-text',
                'title'     => $this->translate('Icinga Documentation')
            )
        ) ?>
    </div>
    <h2><?= $this->translate('Loaded modules') ?></h2>
    <table class="table-row-selectable common-table" data-base-target="_next">
    <thead>
        <tr>
            <th><?= $this->translate('Name') ?></th>
            <th><?= $this->translate('Version') ?></th>
        </tr>
    </thead>
    <tbody>
<?php foreach ($modules as $module): ?>
    <tr>
        <td>
        <?php if ($this->hasPermission('config/modules')): ?>
            <?= $this->qlink(
                $module->getName(),
                'config/module/',
                array('name' => $module->getName()),
                array('title' => sprintf($this->translate('Show the overview of the %s module'), $module->getName()))
            ) ?>
        <?php else: ?>
            <?= $this->escape($module->getName()) ?>
        <?php endif ?>
        <td>
            <?= $this->escape($module->getVersion()) ?>
        </td>
    </tr>
<?php endforeach ?>
    </tbody>
    </table>
</div>
