/*! Icinga Web 2 | (c) 2019 Icinga Development Team | GPLv2+ */

@color-ok: #CCFFD7;
@color-critical: #FE5566;
@color-critical-handled: #FF99AA;
@color-warning: #EAD010;
@color-warning-handled: #FFF399;
@color-unknown: #7791E0;
@color-unknown-handled: #B8C6FF;
@color-unreachable: @color-unknown;
@color-unreachable-handled: @color-unknown-handled;
@color-pending: #000000;

/* Adapt font color to match handled / unhandled states and maintain readability with text-shadows */
.badge {
  font-weight: 800;
}

.state-ok,
.state-up {
  color: black;
}
.state-warning,
.state-critical,
.state-down,
.state-unknown {
  &.handled {
    color: black;
  }
}
.state-warning,
.state-critical,
.state-down,
.state-unknown {
  color: white;
}
.state-warning:not(.handled) {
  text-shadow: 0 0 3px mix(#000000, @color-warning, 60);
}
.state-critical:not(.handled),
.state-down:not(.handled) {
  text-shadow: 0 0 1px mix(#000000, @color-critical, 20);
}
.state-unknown:not(.handled),
.state-unreachable:not(.handled) {
  text-shadow: 0 0 1px mix(#000000, @color-unknown, 20);
}

.processinfo .process > div.backend-running {
  color: @text-color;
}
