/*! Icinga Web 2 | (c) 2015 Icinga Development Team | GPLv2+ */

// TODO(el): Rename .filter to .filter-control

// Hide auto sumbit info in controls
.controls .autosubmit-info {
  display: none;
}


// Backend selection control in user and group list views
.backend-selection {
  float: left;

  .control-label-group, select {
    display: inline-block;
  }

  select {
    margin-left: .5em;
  }
}

.controls input.search,
input.search {
  .transition(border 0.3s ease);
  .transition(background-image 0.2s ease);

  background-image: url(../img/icons/search.png);
  background-repeat: no-repeat;
  background-size: 1em 1em;
  background-position: .25em center;
  outline: none;
  padding-left: 1.5em;
  width: 20em;

  &:focus {
    background-image: url(../img/icons/search_icinga_blue.png);
  }

  &:focus:not([readonly]) {
    border-color: @icinga-blue;
  }
}

.backend-selection,
.pagination-control,
.selection-info,
.sort-controls-container {
  margin-bottom: 0.5em;
}

.filter {
  // Display filter control on a new line
  clear: both;
  margin: .5em 0;

  > a {
    color: @icinga-blue;
    padding: .5em;
    line-height: 1;
  }

  > a > i {
    text-align: center;
     &:before {
       margin-right: 0;
     }
  }

  .form input {
    padding: @vertical-padding @vertical-padding;
  }
}

.controls .filter {
  form .search {
    height: 2em;
  }
}

.controls .button-link {
  height: 2em;
}

.limiter-control > select {
  margin-left: .5em;
}

.pagination-control {
  // Display the pagination-control on a new line
  clear: both;
  float: left;

  li {
    line-height: 1;

    &.active {
      border-bottom: 2px solid @icinga-blue;

      > a,
      > a:hover {
        color: @icinga-blue;
        /* Compensate border-bottom: 2px */
        margin-bottom: -2px;
      }

      > a:hover {
        background: none;
        cursor: default;
        text-decoration: none;
      }
    }

    &.disabled {
      color: @gray-light;
      cursor: no-drop;
    }

    > a,
    > span {
      padding: 0.5em;
    }
    > a:hover {
      background-color: @gray-lighter;
      text-decoration: none;
    }
  }

  .previous-page {
    padding-left: 0;
  }

  .next-page {
    padding-right: 0;
  }
}

// Multi-selection info
.selection-info {
  float: right;
  font-size: @font-size-small;

  &:hover {
    cursor: help;
  }
}

.sort-control {
  label {
    width: auto;
    margin-right: 0.5em;
  }

  select[name=sort] {
    width: 12em;
    margin-left: 0;
  }

  select[name=dir] {
    width: 8em;
    margin-left: 0;
  }
}

.sort-controls-container {
  clear: right;
  float: right;
  display: flex;

  > *:not(:last-child) {
    margin-right: .5em;
  }
}

.sort-direction-control {
  margin-left: 0.25em;
  width: 1em;

  .spinner {
    line-height: 1;
  }
}

.controls {
  .control-label-group {
    margin-top: 0;
    margin-bottom: 0;
    line-height: 1.5em;
    padding-top: 0.25em;
    padding-bottom: 0.25em;
  }

  input,
  select {
    padding: .5em;
    max-width: 16em;
  }

  select {
    padding-right: 1.526em;
    margin-top: 0;
    margin-bottom: 0;
    /* compensate inconsistent select height calculations */
    line-height: 1;
    max-height: 2em;
  }
}
